Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Microsoft.Devices.Sensors

Namespace ViewHorizontalCoordinates
	Partial Public Class MainPage
		Inherits PhoneApplicationPage

		Private PIXELS_PER_DEGREE As Double = 15
		Private motion As Motion
		Private coordinateProjection As New HorizontalCoordinateProjection
		Private lineBrush As Brush = New SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 128, 128, 255))
		Private textBrush As Brush = New SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 192, 192, 255))
        Private _width, _height As Double
        Private compass() As String = {"North", "NE", "East", "SE", "South", "SW", "West", "NW"}
        Private elementsCreated As Boolean

        Public Sub New()
            InitializeComponent()

            If Motion.IsSupported Then
                motion = New Motion
                motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(100)
                AddHandler motion.CurrentValueChanged, AddressOf OnMotionCurrentValueChanged
            End If

            AddHandler Loaded, Sub(sender, args)
                                   _width = ContentPanel.ActualWidth
                                   _height = ContentPanel.ActualHeight
                               End Sub
        End Sub

        Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
            If motion IsNot Nothing Then
                Try
                    motion.Start()
                Catch
                End Try
            End If
            MyBase.OnNavigatedTo(args)
        End Sub

        Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
            If motion IsNot Nothing Then
                motion.Stop()
            End If
            MyBase.OnNavigatedFrom(args)
        End Sub

        Private Sub OnMotionCurrentValueChanged(ByVal sender As Object, ByVal args As SensorReadingEventArgs(Of MotionReading))
            ' Rotate the canvas
            ' Draw azimuth lines
            ' Draw altitude lines
            ' Draw text for altitude angles
            ' Draw text for azimuth compass points
            Me.Dispatcher.BeginInvoke(Sub()
                                          Dim matrix As Microsoft.Xna.Framework.Matrix = args.SensorReading.Attitude.RotationMatrix
                                          Dim viewCenter As HorizontalCoordinate = HorizontalCoordinate.FromMotionMatrix(matrix)
                                          coordinateProjection.SetViewCenter(viewCenter)
                                          rotate.Angle = -viewCenter.Tilt
                                          Dim childIndex As Integer = 0
                                          For azimuth As Double = 0 To 359 Step 15
                                              For altitude As Double = -90 To 89 Step 15
                                                  Dim coord1 As New HorizontalCoordinate(azimuth, altitude)
                                                  Dim coord2 As New HorizontalCoordinate(azimuth, altitude + 15)
                                                  DrawLine(childIndex, coord1, coord2)
                                              Next altitude
                                          Next azimuth
                                          For altitude As Double = -75 To 89 Step 15
                                              For azimuth As Double = 0 To 359 Step 15
                                                  Dim coord1 As New HorizontalCoordinate(azimuth, altitude)
                                                  Dim coord2 As New HorizontalCoordinate(azimuth + 15, altitude)
                                                  DrawLine(childIndex, coord1, coord2)
                                              Next azimuth
                                          Next altitude
                                          For altitude As Double = -75 To 89 Step 15
                                              Dim coord As New HorizontalCoordinate(viewCenter.Azimuth, altitude)
                                              DrawText(childIndex, altitude.ToString, coord)
                                          Next altitude
                                          For azimuth As Double = 0 To 359 Step 45
                                              Dim altitude As Double = Math.Min(80, Math.Max(-80, viewCenter.Altitude))
                                              Dim coord As New HorizontalCoordinate(azimuth, altitude)
                                              DrawText(childIndex, compass(CInt(azimuth / 45)), coord)
                                          Next azimuth
                                          elementsCreated = True
                                      End Sub)
        End Sub

        Private Sub DrawLine(ByRef childIndex As Integer, ByVal coord1 As HorizontalCoordinate, ByVal coord2 As HorizontalCoordinate)
            Dim point1 As Point = CalculateScreenPoint(coord1)
            Dim point2 As Point = CalculateScreenPoint(coord2)

            ' If points are out of range, move them offscreen
            If Double.IsNaN(point1.X) OrElse Double.IsNaN(point2.X) Then
                point2 = New Point(-1000, -1000)
                point1 = point2
            End If

            Dim line As Line = Nothing

            ' Create Line first time through
            If Not elementsCreated Then
                line = New Line With {.Stroke = lineBrush, .StrokeThickness = 3}
                canvas.Children.Add(line)
                ' Get created Line otherwise
            Else
                line = TryCast(canvas.Children(childIndex), Line)
                childIndex += 1
            End If

            line.X1 = point1.X
            line.Y1 = point1.Y
            line.X2 = point2.X
            line.Y2 = point2.Y
        End Sub

        Private Sub DrawText(ByRef childIndex As Integer, ByVal str As String, ByVal coord As HorizontalCoordinate)
            Dim point As Point = CalculateScreenPoint(coord)

            ' If point is out of range, move them offscreen
            If Double.IsNaN(point.X) Then
                point = New Point(-1000, -1000)
            End If

            Dim txtblk As TextBlock = Nothing

            ' Create TextBlock first time through
            If Not elementsCreated Then
                txtblk = New TextBlock With {.Text = str, .FontSize = 48, .Foreground = textBrush}
                canvas.Children.Add(txtblk)
                ' Get created TextBlock otherwise
            Else
                txtblk = TryCast(canvas.Children(childIndex), TextBlock)
                childIndex += 1
            End If

            Canvas.SetLeft(txtblk, point.X - txtblk.ActualWidth / 2)
            Canvas.SetTop(txtblk, point.Y - txtblk.ActualHeight / 2)
        End Sub

        Private Function CalculateScreenPoint(ByVal horizontalCoordinate As HorizontalCoordinate) As Point
            Dim horzAngle, vertAngle As Double
            coordinateProjection.GetAngleOffsets(horizontalCoordinate, horzAngle, vertAngle)

            ' Use NaN to indicate points clearly out of range of the screen
            Dim x As Single = Single.NaN
            Dim y As Single = Single.NaN

            If horzAngle > -90 AndAlso horzAngle < 90 AndAlso vertAngle > -90 AndAlso vertAngle < 90 Then
                x = CSng(_width / 2 + PIXELS_PER_DEGREE * horzAngle)
                y = CSng(_height / 2 + PIXELS_PER_DEGREE * vertAngle)
            End If
            Return New Point(x, y)
        End Function
	End Class
End Namespace
